@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) 2004 Wim Taymans <wim@fluendo.com>
+ * Copyright (C) 2011 Sebastian Dröge <sebastian.droege@collabora.co.uk>
  *
  * gstiterator.h: Base class for iterating datastructures.
  *
@@ -15,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -31,64 +32,83 @@
  * Various GStreamer objects provide access to their internal structures using
  * an iterator.
  *
- * In general, whenever calling a GstIterator function results in your code
- * receiving a refcounted object, the refcount for that object will have been
- * increased.  Your code is responsible for unrefing that object after use.
+ * Note that if calling a GstIterator function results in your code receiving
+ * a refcounted object (with, say, g_value_get_object()), the refcount for that
+ * object will not be increased. Your code is responsible for taking a reference
+ * if it wants to continue using it later.
  *
  * The basic use pattern of an iterator is as follows:
- *
- * <example>
- * <title>Using an iterator</title>
- *   <programlisting>
- *    it = _get_iterator(object);
- *    done = FALSE;
- *    while (!done) {
- *      switch (gst_iterator_next (it, &amp;item)) {
- *        case GST_ITERATOR_OK:
- *          ... use/change item here...
- *          gst_object_unref (item);
- *          break;
- *        case GST_ITERATOR_RESYNC:
- *          ...rollback changes to items...
- *          gst_iterator_resync (it);
- *          break;
- *        case GST_ITERATOR_ERROR:
- *          ...wrong parameters were given...
- *          done = TRUE;
- *          break;
- *        case GST_ITERATOR_DONE:
- *          done = TRUE;
- *          break;
- *      }
- *    }
- *    gst_iterator_free (it);
- *   </programlisting>
- * </example>
- *
- * Last reviewed on 2009-06-16 (0.10.24)
+ * |[
+ *   GstIterator *it = _get_iterator(object);
+ *   GValue item = G_VALUE_INIT;
+ *   done = FALSE;
+ *   while (!done) {
+ *     switch (gst_iterator_next (it, &amp;item)) {
+ *       case GST_ITERATOR_OK:
+ *         ...get/use/change item here...
+ *         g_value_reset (&amp;item);
+ *         break;
+ *       case GST_ITERATOR_RESYNC:
+ *         ...rollback changes to items...
+ *         gst_iterator_resync (it);
+ *         break;
+ *       case GST_ITERATOR_ERROR:
+ *         ...wrong parameters were given...
+ *         done = TRUE;
+ *         break;
+ *       case GST_ITERATOR_DONE:
+ *         done = TRUE;
+ *         break;
+ *     }
+ *   }
+ *   g_value_unset (&amp;item);
+ *   gst_iterator_free (it);
+ * ]|
  */
 
 #include "gst_private.h"
 #include <gst/gstiterator.h>
 
-/* FIXME 0.11: Store the size inside the iterator, use GSlice for allocation
- * and let gst_iterator_free() free the memory while the free-func only frees
- * additional resources (maybe call it finalize?).
+/**
+ * gst_iterator_copy:
+ * @it: a #GstIterator
+ *
+ * Copy the iterator and its state.
+ *
+ * Returns: a new copy of @it.
  */
+GstIterator *
+gst_iterator_copy (const GstIterator * it)
+{
+  GstIterator *copy;
+
+  copy = g_slice_copy (it->size, it);
+  if (it->copy)
+    it->copy (it, copy);
+
+  return copy;
+}
+
+G_DEFINE_BOXED_TYPE (GstIterator, gst_iterator,
+    (GBoxedCopyFunc) gst_iterator_copy, (GBoxedFreeFunc) gst_iterator_free);
 
 static void
 gst_iterator_init (GstIterator * it,
+    guint size,
     GType type,
     GMutex * lock,
     guint32 * master_cookie,
+    GstIteratorCopyFunction copy,
     GstIteratorNextFunction next,
     GstIteratorItemFunction item,
     GstIteratorResyncFunction resync, GstIteratorFreeFunction free)
 {
+  it->size = size;
   it->type = type;
   it->lock = lock;
   it->master_cookie = master_cookie;
   it->cookie = *master_cookie;
+  it->copy = copy;
   it->next = next;
   it->item = item;
   it->resync = resync;
@@ -97,12 +117,13 @@ gst_iterator_init (GstIterator * it,
 }
 
 /**
- * gst_iterator_new:
+ * gst_iterator_new: (skip)
  * @size: the size of the iterator structure
  * @type: #GType of children
  * @lock: pointer to a #GMutex.
  * @master_cookie: pointer to a guint32 that is changed when the items in the
  *    iterator changed.
+ * @copy: copy function
  * @next: function to get next item
  * @item: function to call on each item retrieved
  * @resync: function to resync the iterator
@@ -123,6 +144,7 @@ gst_iterator_new (guint size,
     GType type,
     GMutex * lock,
     guint32 * master_cookie,
+    GstIteratorCopyFunction copy,
     GstIteratorNextFunction next,
     GstIteratorItemFunction item,
     GstIteratorResyncFunction resync, GstIteratorFreeFunction free)
@@ -136,9 +158,9 @@ gst_iterator_new (guint size,
   g_return_val_if_fail (resync != NULL, NULL);
   g_return_val_if_fail (free != NULL, NULL);
 
-  result = g_malloc (size);
-  gst_iterator_init (result, type, lock, master_cookie, next, item, resync,
-      free);
+  result = g_slice_alloc0 (size);
+  gst_iterator_init (result, size, type, lock, master_cookie, copy, next, item,
+      resync, free);
 
   return result;
 }
@@ -149,21 +171,33 @@ gst_iterator_new (guint size,
 typedef struct _GstListIterator
 {
   GstIterator iterator;
-  gpointer owner;
+  GObject *owner;
   GList **orig;
   GList *list;                  /* pointer in list */
-  GstIteratorDisposeFunction freefunc;
+
+  void (*set_value) (GValue * value, gpointer item);
 } GstListIterator;
 
-static GstIteratorResult
-gst_list_iterator_next (GstListIterator * it, gpointer * elem)
+static void
+gst_list_iterator_copy (const GstListIterator * it, GstListIterator * copy)
 {
+  if (copy->owner)
+    g_object_ref (copy->owner);
+}
+
+static GstIteratorResult
+gst_list_iterator_next (GstListIterator * it, GValue * elem)
+{
+  gpointer data;
+
   if (it->list == NULL)
     return GST_ITERATOR_DONE;
 
-  *elem = it->list->data;
+  data = it->list->data;
   it->list = g_list_next (it->list);
 
+  it->set_value (elem, data);
+
   return GST_ITERATOR_OK;
 }
 
@@ -176,22 +210,19 @@ gst_list_iterator_resync (GstListIterator * it)
 static void
 gst_list_iterator_free (GstListIterator * it)
 {
-  if (it->freefunc) {
-    it->freefunc (it->owner);
-  }
-  g_free (it);
+  if (it->owner)
+    g_object_unref (it->owner);
 }
 
 /**
- * gst_iterator_new_list:
+ * gst_iterator_new_list: (skip)
  * @type: #GType of elements
  * @lock: pointer to a #GMutex protecting the list.
  * @master_cookie: pointer to a guint32 that is incremented when the list
  *     is changed.
  * @list: pointer to the list
  * @owner: object owning the list
- * @item: function to call for each item
- * @free: function to call when the iterator is freed
+ * @item: function to call on each item retrieved
  *
  * Create a new iterator designed for iterating @list.
  *
@@ -201,46 +232,52 @@ gst_list_iterator_free (GstListIterator * it)
  * The iterator will use @lock to retrieve the next item of the list and it
  * will then call the @item function before releasing @lock again.
  *
- * The @item function usualy makes sure that the item remains alive while
- * @lock is released and the application is using the item. The application is
- * responsible for freeing/unreffing the item after usage as explained in
- * gst_iterator_next().
- *
  * When a concurrent update to the list is performed, usually by @owner while
  * holding @lock, @master_cookie will be updated. The iterator implementation
  * will notice the update of the cookie and will return %GST_ITERATOR_RESYNC to
  * the user of the iterator in the next call to gst_iterator_next().
  *
- * @owner will be passed to the @free function when the iterator is freed.
- *
  * Returns: the new #GstIterator for @list.
  *
  * MT safe.
  */
 GstIterator *
 gst_iterator_new_list (GType type,
-    GMutex * lock,
-    guint32 * master_cookie,
-    GList ** list,
-    gpointer owner,
-    GstIteratorItemFunction item, GstIteratorDisposeFunction free)
+    GMutex * lock, guint32 * master_cookie, GList ** list, GObject * owner,
+    GstIteratorItemFunction item)
 {
   GstListIterator *result;
+  gpointer set_value;
+
+  if (g_type_is_a (type, G_TYPE_OBJECT)) {
+    set_value = g_value_set_object;
+  } else if (g_type_is_a (type, G_TYPE_BOXED)) {
+    set_value = g_value_set_boxed;
+  } else if (g_type_is_a (type, G_TYPE_POINTER)) {
+    set_value = g_value_set_pointer;
+  } else if (g_type_is_a (type, G_TYPE_STRING)) {
+    set_value = g_value_set_string;
+  } else {
+    g_critical ("List iterators can only be created for lists containing "
+        "instances of GObject, boxed types, pointer types and strings");
+    return NULL;
+  }
 
   /* no need to lock, nothing can change here */
   result = (GstListIterator *) gst_iterator_new (sizeof (GstListIterator),
       type,
       lock,
       master_cookie,
+      (GstIteratorCopyFunction) gst_list_iterator_copy,
       (GstIteratorNextFunction) gst_list_iterator_next,
       (GstIteratorItemFunction) item,
       (GstIteratorResyncFunction) gst_list_iterator_resync,
       (GstIteratorFreeFunction) gst_list_iterator_free);
 
-  result->owner = owner;
+  result->owner = owner ? g_object_ref (owner) : NULL;
   result->orig = list;
   result->list = *list;
-  result->freefunc = free;
+  result->set_value = set_value;
 
   return GST_ITERATOR (result);
 }
@@ -257,14 +294,15 @@ gst_iterator_pop (GstIterator * it)
 /**
  * gst_iterator_next:
  * @it: The #GstIterator to iterate
- * @elem: pointer to hold next element
+ * @elem: (out caller-allocates): pointer to hold next element
  *
  * Get the next item from the iterator in @elem. 
  *
  * Only when this function returns %GST_ITERATOR_OK, @elem will contain a valid
- * value. For iterators that return refcounted objects, the returned object
- * will have its refcount increased and should therefore be unreffed after
- * usage.
+ * value. @elem must have been initialized to the type of the iterator or
+ * initialized to zeroes with g_value_unset(). The caller is responsible for
+ * unsetting or resetting @elem with g_value_unset() or g_value_reset()
+ * after usage.
  *
  * When this function returns %GST_ITERATOR_DONE, no more elements can be
  * retrieved from @it.
@@ -275,18 +313,22 @@ gst_iterator_pop (GstIterator * it)
  *
  * A return value of %GST_ITERATOR_ERROR indicates an unrecoverable fatal error.
  *
- * Returns: The result of the iteration. Unref @elem after usage if this
- * is a refcounted object.
+ * Returns: The result of the iteration. Unset @elem after usage.
  *
  * MT safe.
  */
 GstIteratorResult
-gst_iterator_next (GstIterator * it, gpointer * elem)
+gst_iterator_next (GstIterator * it, GValue * elem)
 {
   GstIteratorResult result;
 
   g_return_val_if_fail (it != NULL, GST_ITERATOR_ERROR);
   g_return_val_if_fail (elem != NULL, GST_ITERATOR_ERROR);
+  g_return_val_if_fail (G_VALUE_TYPE (elem) == G_TYPE_INVALID
+      || G_VALUE_HOLDS (elem, it->type), GST_ITERATOR_ERROR);
+
+  if (G_VALUE_TYPE (elem) == G_TYPE_INVALID)
+    g_value_init (elem, it->type);
 
 restart:
   if (it->pushed) {
@@ -312,14 +354,16 @@ restart:
   if (result == GST_ITERATOR_OK && it->item) {
     GstIteratorItem itemres;
 
-    itemres = it->item (it, *elem);
+    itemres = it->item (it, elem);
     switch (itemres) {
       case GST_ITERATOR_ITEM_SKIP:
         if (G_LIKELY (it->lock))
           g_mutex_unlock (it->lock);
+        g_value_reset (elem);
         goto restart;
       case GST_ITERATOR_ITEM_END:
         result = GST_ITERATOR_DONE;
+        g_value_reset (elem);
         break;
       case GST_ITERATOR_ITEM_PASS:
         break;
@@ -376,6 +420,8 @@ gst_iterator_free (GstIterator * it)
   gst_iterator_pop (it);
 
   it->free (it);
+
+  g_slice_free1 (it->size, it);
 }
 
 /**
@@ -409,32 +455,32 @@ typedef struct _GstIteratorFilter
   GstIterator iterator;
   GstIterator *slave;
 
+  GMutex *master_lock;
   GCompareFunc func;
-  gpointer user_data;
+  GValue user_data;
+  gboolean have_user_data;
 } GstIteratorFilter;
 
 static GstIteratorResult
-filter_next (GstIteratorFilter * it, gpointer * elem)
+filter_next (GstIteratorFilter * it, GValue * elem)
 {
   GstIteratorResult result = GST_ITERATOR_ERROR;
   gboolean done = FALSE;
-
-  *elem = NULL;
+  GValue item = { 0, };
 
   while (G_LIKELY (!done)) {
-    gpointer item;
-
     result = gst_iterator_next (it->slave, &item);
     switch (result) {
       case GST_ITERATOR_OK:
-        if (G_LIKELY (GST_ITERATOR (it)->lock))
-          g_mutex_unlock (GST_ITERATOR (it)->lock);
-        if (it->func (item, it->user_data) == 0) {
-          *elem = item;
+        if (G_LIKELY (it->master_lock))
+          g_mutex_unlock (it->master_lock);
+        if (it->func (&item, &it->user_data) == 0) {
+          g_value_copy (&item, elem);
           done = TRUE;
         }
-        if (G_LIKELY (GST_ITERATOR (it)->lock))
-          g_mutex_lock (GST_ITERATOR (it)->lock);
+        g_value_reset (&item);
+        if (G_LIKELY (it->master_lock))
+          g_mutex_lock (it->master_lock);
         break;
       case GST_ITERATOR_RESYNC:
       case GST_ITERATOR_DONE:
@@ -445,48 +491,59 @@ filter_next (GstIteratorFilter * it, gpointer * elem)
         break;
     }
   }
+  g_value_unset (&item);
   return result;
 }
 
+static void
+filter_copy (const GstIteratorFilter * it, GstIteratorFilter * copy)
+{
+  copy->slave = gst_iterator_copy (it->slave);
+  copy->master_lock = copy->slave->lock ? copy->slave->lock : it->master_lock;
+  copy->slave->lock = NULL;
+
+  if (it->have_user_data) {
+    memset (&copy->user_data, 0, sizeof (copy->user_data));
+    g_value_init (&copy->user_data, G_VALUE_TYPE (&it->user_data));
+    g_value_copy (&it->user_data, &copy->user_data);
+  }
+}
+
 static void
 filter_resync (GstIteratorFilter * it)
 {
   gst_iterator_resync (it->slave);
 }
 
-static void
-filter_uninit (GstIteratorFilter * it)
-{
-  it->slave->lock = GST_ITERATOR (it)->lock;
-}
-
 static void
 filter_free (GstIteratorFilter * it)
 {
-  filter_uninit (it);
+  if (it->have_user_data)
+    g_value_unset (&it->user_data);
   gst_iterator_free (it->slave);
-  g_free (it);
 }
 
 /**
  * gst_iterator_filter:
  * @it: The #GstIterator to filter
- * @func: the compare function to select elements
- * @user_data: user data passed to the compare function
+ * @func: (scope call): the compare function to select elements
+ * @user_data: (closure): user data passed to the compare function
  *
  * Create a new iterator from an existing iterator. The new iterator
  * will only return those elements that match the given compare function @func.
- * @func should return 0 for elements that should be included
- * in the iterator.
+ * The first parameter that is passed to @func is the #GValue of the current
+ * iterator element and the second parameter is @user_data. @func should
+ * return 0 for elements that should be included in the filtered iterator.
  *
  * When this iterator is freed, @it will also be freed.
  *
- * Returns: a new #GstIterator.
+ * Returns: (transfer full): a new #GstIterator.
  *
  * MT safe.
  */
 GstIterator *
-gst_iterator_filter (GstIterator * it, GCompareFunc func, gpointer user_data)
+gst_iterator_filter (GstIterator * it, GCompareFunc func,
+    const GValue * user_data)
 {
   GstIteratorFilter *result;
 
@@ -495,13 +552,22 @@ gst_iterator_filter (GstIterator * it, GCompareFunc func, gpointer user_data)
 
   result = (GstIteratorFilter *) gst_iterator_new (sizeof (GstIteratorFilter),
       it->type, it->lock, it->master_cookie,
+      (GstIteratorCopyFunction) filter_copy,
       (GstIteratorNextFunction) filter_next,
       (GstIteratorItemFunction) NULL,
       (GstIteratorResyncFunction) filter_resync,
       (GstIteratorFreeFunction) filter_free);
+
+  result->master_lock = it->lock;
   it->lock = NULL;
   result->func = func;
-  result->user_data = user_data;
+  if (user_data) {
+    g_value_init (&result->user_data, G_VALUE_TYPE (user_data));
+    g_value_copy (user_data, &result->user_data);
+    result->have_user_data = TRUE;
+  } else {
+    result->have_user_data = FALSE;
+  }
   result->slave = it;
 
   return GST_ITERATOR (result);
@@ -510,21 +576,20 @@ gst_iterator_filter (GstIterator * it, GCompareFunc func, gpointer user_data)
 /**
  * gst_iterator_fold:
  * @it: The #GstIterator to fold over
- * @func: the fold function
+ * @func: (scope call): the fold function
  * @ret: the seed value passed to the fold function
- * @user_data: user data passed to the fold function
+ * @user_data: (closure): user data passed to the fold function
  *
  * Folds @func over the elements of @iter. That is to say, @func will be called
  * as @func (object, @ret, @user_data) for each object in @it. The normal use
  * of this procedure is to accumulate the results of operating on the objects in
- * @ret.  If object is a refcounted object its refcount will be increased 
- * before @func is called, and it should be unrefed after use in @func.
+ * @ret.
  *
  * This procedure can be used (and is used internally) to implement the
  * gst_iterator_foreach() and gst_iterator_find_custom() operations.
  *
- * The fold will proceed as long as @func returns TRUE. When the iterator has no
- * more arguments, %GST_ITERATOR_DONE will be returned. If @func returns FALSE,
+ * The fold will proceed as long as @func returns %TRUE. When the iterator has no
+ * more arguments, %GST_ITERATOR_DONE will be returned. If @func returns %FALSE,
  * the fold will stop, and %GST_ITERATOR_OK will be returned. Errors or resyncs
  * will cause fold to return %GST_ITERATOR_ERROR or %GST_ITERATOR_RESYNC as
  * appropriate.
@@ -539,18 +604,18 @@ GstIteratorResult
 gst_iterator_fold (GstIterator * it, GstIteratorFoldFunction func,
     GValue * ret, gpointer user_data)
 {
-  gpointer item;
+  GValue item = { 0, };
   GstIteratorResult result;
 
   while (1) {
     result = gst_iterator_next (it, &item);
     switch (result) {
       case GST_ITERATOR_OK:
-        /* FIXME: is there a way to ref/unref items? */
-        if (!func (item, ret, user_data))
+        if (!func (&item, ret, user_data))
           goto fold_done;
-        else
-          break;
+
+        g_value_reset (&item);
+        break;
       case GST_ITERATOR_RESYNC:
       case GST_ITERATOR_ERROR:
         goto fold_done;
@@ -560,17 +625,19 @@ gst_iterator_fold (GstIterator * it, GstIteratorFoldFunction func,
   }
 
 fold_done:
+  g_value_unset (&item);
+
   return result;
 }
 
 typedef struct
 {
-  GFunc func;
+  GstIteratorForeachFunction func;
   gpointer user_data;
 } ForeachFoldData;
 
 static gboolean
-foreach_fold_func (gpointer item, GValue * unused, ForeachFoldData * data)
+foreach_fold_func (const GValue * item, GValue * unused, ForeachFoldData * data)
 {
   data->func (item, data->user_data);
   return TRUE;
@@ -579,13 +646,11 @@ foreach_fold_func (gpointer item, GValue * unused, ForeachFoldData * data)
 /**
  * gst_iterator_foreach:
  * @it: The #GstIterator to iterate
- * @func: the function to call for each element.
- * @user_data: user data passed to the function
+ * @func: (scope call): the function to call for each element.
+ * @user_data: (closure): user data passed to the function
  *
  * Iterate over all element of @it and call the given function @func for
- * each element.  As in gst_iterator_fold(), the refcount of a refcounted 
- * object will be increased before @func is called, and should be unrefed
- * after use.
+ * each element.
  *
  * Returns: the result call to gst_iterator_fold(). The iterator will not be
  * freed.
@@ -593,7 +658,8 @@ foreach_fold_func (gpointer item, GValue * unused, ForeachFoldData * data)
  * MT safe.
  */
 GstIteratorResult
-gst_iterator_foreach (GstIterator * it, GFunc func, gpointer user_data)
+gst_iterator_foreach (GstIterator * it, GstIteratorForeachFunction func,
+    gpointer user_data)
 {
   ForeachFoldData data;
 
@@ -608,13 +674,16 @@ typedef struct
 {
   GCompareFunc func;
   gpointer user_data;
+  gboolean found;
 } FindCustomFoldData;
 
 static gboolean
-find_custom_fold_func (gpointer item, GValue * ret, FindCustomFoldData * data)
+find_custom_fold_func (const GValue * item, GValue * ret,
+    FindCustomFoldData * data)
 {
   if (data->func (item, data->user_data) == 0) {
-    g_value_set_pointer (ret, item);
+    data->found = TRUE;
+    g_value_copy (item, ret);
     return FALSE;
   } else {
     return TRUE;
@@ -624,68 +693,86 @@ find_custom_fold_func (gpointer item, GValue * ret, FindCustomFoldData * data)
 /**
  * gst_iterator_find_custom:
  * @it: The #GstIterator to iterate
- * @func: the compare function to use
- * @user_data: user data passed to the compare function
+ * @func: (scope call): the compare function to use
+ * @elem: (out): pointer to a #GValue where to store the result
+ * @user_data: (closure): user data passed to the compare function
  *
  * Find the first element in @it that matches the compare function @func.
- * @func should return 0 when the element is found.  As in gst_iterator_fold(),
- * the refcount of a refcounted object will be increased before @func is 
- * called, and should be unrefed after use.
+ * @func should return 0 when the element is found. The first parameter
+ * to @func will be the current element of the iterator and the
+ * second parameter will be @user_data.
+ * The result will be stored in @elem if a result is found.
  *
  * The iterator will not be freed.
  *
- * This function will return NULL if an error or resync happened to
- * the iterator.
+ * This function will return %FALSE if an error happened to the iterator
+ * or if the element wasn't found.
  *
- * Returns: The element in the iterator that matches the compare
- * function or NULL when no element matched.
+ * Returns: Returns %TRUE if the element was found, else %FALSE.
  *
  * MT safe.
  */
-gpointer
+gboolean
 gst_iterator_find_custom (GstIterator * it, GCompareFunc func,
-    gpointer user_data)
+    GValue * elem, gpointer user_data)
 {
-  GValue ret = { 0, };
   GstIteratorResult res;
   FindCustomFoldData data;
 
-  g_value_init (&ret, G_TYPE_POINTER);
+  g_return_val_if_fail (G_VALUE_TYPE (elem) == G_TYPE_INVALID
+      || G_VALUE_HOLDS (elem, it->type), GST_ITERATOR_ERROR);
+
+  if (G_VALUE_TYPE (elem) == G_TYPE_INVALID)
+    g_value_init (elem, it->type);
+
   data.func = func;
   data.user_data = user_data;
+  data.found = FALSE;
 
-  /* FIXME, we totally ignore RESYNC and return NULL so that the
-   * app does not know if the element was not found or a resync happened */
-  res =
-      gst_iterator_fold (it, (GstIteratorFoldFunction) find_custom_fold_func,
-      &ret, &data);
+  do {
+    res =
+        gst_iterator_fold (it, (GstIteratorFoldFunction) find_custom_fold_func,
+        elem, &data);
+    if (res == GST_ITERATOR_RESYNC)
+      gst_iterator_resync (it);
+  } while (res == GST_ITERATOR_RESYNC);
 
-  /* no need to unset, it's just a pointer */
-  return g_value_get_pointer (&ret);
+  if (!data.found)
+    g_value_unset (elem);
+
+  return data.found;
 }
 
 typedef struct
 {
   GstIterator parent;
-  gpointer object;
-  GstCopyFunction copy;
-  GFreeFunc free;
+  GValue object;
   gboolean visited;
+  gboolean empty;
 } GstSingleObjectIterator;
 
 static guint32 _single_object_dummy_cookie = 0;
 
-static GstIteratorResult
-gst_single_object_iterator_iterator_next (GstSingleObjectIterator * it,
-    gpointer * result)
+static void
+gst_single_object_iterator_copy (const GstSingleObjectIterator * it,
+    GstSingleObjectIterator * copy)
 {
-  if (it->visited || !it->object) {
-    *result = NULL;
-    return GST_ITERATOR_DONE;
+  if (!it->empty) {
+    memset (&copy->object, 0, sizeof (copy->object));
+    g_value_init (&copy->object, it->parent.type);
+    g_value_copy (&it->object, &copy->object);
   }
+}
 
-  *result = it->copy (it->object);
+static GstIteratorResult
+gst_single_object_iterator_next (GstSingleObjectIterator * it, GValue * result)
+{
+  if (it->visited || it->empty)
+    return GST_ITERATOR_DONE;
+
+  g_value_copy (&it->object, result);
   it->visited = TRUE;
+
   return GST_ITERATOR_OK;
 }
 
@@ -698,17 +785,14 @@ gst_single_object_iterator_resync (GstSingleObjectIterator * it)
 static void
 gst_single_object_iterator_free (GstSingleObjectIterator * it)
 {
-  if (it->object)
-    it->free (it->object);
-  g_free (it);
+  if (!it->empty)
+    g_value_unset (&it->object);
 }
 
 /**
  * gst_iterator_new_single:
  * @type: #GType of the passed object
  * @object: object that this iterator should return
- * @copy: Function that returns a copy of @object or increases its refcount
- * @free: Function to be called for freeing @object
  *
  * This #GstIterator is a convenient iterator for the common
  * case where a #GstIterator needs to be returned but only
@@ -716,29 +800,29 @@ gst_single_object_iterator_free (GstSingleObjectIterator * it)
  * for the #GstPadIterIntLinkFunction.
  *
  * Returns: the new #GstIterator for @object.
- *
- * Since: 0.10.25
  */
 GstIterator *
-gst_iterator_new_single (GType type, gpointer object, GstCopyFunction copy,
-    GFreeFunc free)
+gst_iterator_new_single (GType type, const GValue * object)
 {
   GstSingleObjectIterator *result;
 
-  g_return_val_if_fail (copy != NULL, NULL);
-  g_return_val_if_fail (free != NULL, NULL);
-
   result = (GstSingleObjectIterator *)
       gst_iterator_new (sizeof (GstSingleObjectIterator),
       type, NULL, &_single_object_dummy_cookie,
-      (GstIteratorNextFunction) gst_single_object_iterator_iterator_next, NULL,
+      (GstIteratorCopyFunction) gst_single_object_iterator_copy,
+      (GstIteratorNextFunction) gst_single_object_iterator_next,
+      (GstIteratorItemFunction) NULL,
       (GstIteratorResyncFunction) gst_single_object_iterator_resync,
       (GstIteratorFreeFunction) gst_single_object_iterator_free);
 
-  result->object = (object) ? copy (object) : NULL;
-  result->copy = copy;
-  result->free = free;
+  if (object) {
+    g_value_init (&result->object, type);
+    g_value_copy (object, &result->object);
+    result->empty = FALSE;
+  } else {
+    result->empty = TRUE;
+  }
   result->visited = FALSE;
 
-  return (GstIterator *) result;
+  return GST_ITERATOR (result);
 }
