@@ -1015,8 +1015,11 @@ gst_check_object_destroyed_on_unref (gpointer object_to_unref)
 }
 
 /* For ABI compatibility with GStreamer < 1.5 */
+/* *INDENT-OFF* */
 void
-_fail_unless (int result, const char *file, int line, const char *expr, ...);
+_fail_unless (int result, const char *file, int line, const char *expr, ...)
+G_GNUC_PRINTF (4, 5);
+/* *INDENT-ON* */
 
 void
 _fail_unless (int result, const char *file, int line, const char *expr, ...)
